#!/usr/bin/perl
#
# b52fc.pl
#

require 'b5to2022.tbl';


$tempfile = "/tmp/perl.$$";

open(SORT_FH, "| sort > $tempfile") || die 'Cannot pipe to "sort".';

#
# This perl script converts the file cefdb of the KanjiBase for Windows
# package written by Christian Wittern into a form usable by the quail
# input system of MULE.
#
# character set:        Big 5
# quail input system:   four corner
#
# Usage:    perl b52fc.pl < cefdb > 4c-b5.el
#

$[ = 1;                 # set array base to 1
$FS = "\t";             # set field separator
$, = "\t";              # set output field separator
$\ = "\n";              # set output record separator


while (<>) {
    chop;               # strip record separator
    @Fld = split($FS, $_);

    # Now we check whether the current input line has more than three
    # fields. If yes, print each four corner code together with the Big 5
    # character code (the third field) and sort it. All other fields of the
    # database are ignored.
    #
    # All other encodings (C1-...., etc.) will be left out.

    if ($#Fld >= 4) {
        $plane = substr($Fld[3], 2, 1);
        print SORT_FH substr($Fld[4], 1, 4), $Fld[3]
          if($plane eq '0');
    }
}

close(SORT_FH);


# We now read the sorted file and arrange the entries as needed by the
# quail system: each four corner code is followed by a string of Big 5
# encoded characters.

open(TMP_FH, $tempfile);
$lastFld = '';
$outFld = '';


# The quail package header.

print "(require 'quail)";
print '(quail-define-package "4c-b5" "4c-b5"';
print ' t';
print ' "Big 5 four corner input package';
print 'mechanically extracted from the cefdb database of the';
print 'KanjiBase for Windows package written by Christian Wittern."';
print " '(";
print '  (">" . quail-next-candidate-block)';
print '  ("<" . quail-prev-candidate-block)';
print '  (" " . quail-select-current)';
print '  )';
print ' nil nil)';
print '';
print ';;';


# The data section.

while (<TMP_FH>) {
    chop;
    @Fld = split($FS, $_);

    if ($Fld[1] eq $lastFld) {
        $outFld .= $Fld[2];
    }
    else {
        $outFld = &conv($outFld);
        $prefix = ($char_count == 1) ? 'qd  ' : 'qdv ';

        print '('.$prefix.'"'.$lastFld.'"', 
              '"'.$outFld.'")' if ($lastFld ne '');

        $lastFld = $Fld[1];
        $outFld = $Fld[2];
    }
}

$outFld = &conv($outFld);
$prefix = ($char_count == 1) ? 'qd  ' : 'qdv ';

print '('.$prefix.'"'.$lastFld.'"',         # the last line of data
      '"'.$outFld.'")' if ($lastFld ne '');


# The postamble.

print '';
print '(quail-setup-current-package)';

close(TMP_FH);
unlink $tempfile;

exit;


# The following subroutine converts the CEF representation of a CNS encoded
# character string into an ISO-2022 string as used by MULE.
#
# Only Big 5 characters are used.

sub conv {
    @CNS_plane_id = ('G' .. 'M');           # ISO 2022 final characters
    @Big5_plane_id = ('0', '1');
    $last_CNS_plane = 0;
    $last_Big5_plane = 0;
    $result = '';
    $char_count = 0;

    while (@_[1] =~ /C(.)-(..)(..)/go) {    # split CNS string char by char
        $char_count++;
        if ($1 == 0) {
            $code = hex("$2" . "$3");
            $code_string = pack("CC", hex($2), hex($3));
            $new_Big5_plane = ($code <= 0xc67e) ? 1 : 2;
            $result .= "\e".'$('."$Big5_plane_id[$new_Big5_plane]"
                         if ($new_Big5_plane != $last_Big5_plane);
            $result .= ($new_Big5_plane == 1) ? 
                         $big5to2022_1{$code_string} :
                         $big5to2022_2{$code_string};
            $last_Big5_plane = $new_Big5_plane;
        }
        else {
            $result .= "\e".'$('."$CNS_plane_id[$1]" 
                         if ($1 != $last_CNS_plane);
            $result .= sprintf("%c%c", hex($2), hex($3));
            $last_Big5_plane = 0;
        }
        $last_CNS_plane = $1;
    }
    $result .= "\e(B";
    return $result;
}

# EOF
